@@ -4,8 +4,8 @@ package NetAddr::IP;
 
 use strict;
 #use diagnostics;
-use NetAddr::IP::Lite 1.02 qw(Zero Ones V4mask V4net);
-use NetAddr::IP::Util qw(
+use NetAddr::IP::Lite 1.07 qw(Zero Ones V4mask V4net);
+use NetAddr::IP::Util 1.04 qw(
 	sub128
 	inet_aton
 	inet_any2n
@@ -31,7 +31,7 @@ require Exporter;
 
 @ISA = qw(Exporter NetAddr::IP::Lite);
 
-$VERSION = do { sprintf " %d.%03d", (q$Revision: 4.7 $ =~ /\d+/g) };
+$VERSION = do { sprintf " %d.%03d", (q$Revision: 4.10 $ =~ /\d+/g) };
 
 =pod
 
@@ -48,12 +48,14 @@ NetAddr::IP - Manages IPv4 and IPv6 addresses and subnets
 	Ones
 	V4mask
 	V4net
-	:aton
+	:aton		DEPRECATED
 	:old_storable
 	:old_nth
   );
 
   my $ip = new NetAddr::IP 'loopback';
+	or
+  my $ip = new_from_aton NetAddr::IP::Lite (inet_aton('127.0.0.1'));
 
   print "The address is ", $ip->addr, " with mask ", $ip->mask, "\n" ;
 
@@ -72,11 +74,15 @@ NetAddr::IP - Manages IPv4 and IPv6 addresses and subnets
   ::FFFF:FFFF                              = V4net();
 
 
-* To accept addresses in the format as returned by inet_aton, invoke the module
-as:
+###### DEPRECATED, will be remove in version 5 ############
+
+  * To accept addresses in the format as returned by 
+  inet_aton, invoke the module as:
 
   use NetAddr::IP qw(:aton);
 
+###### USE new_from_aton instead ##########################
+
 * To enable usage of legacy data files containing NetAddr::IP
 objects stored using the L<Storable> module.
 
@@ -184,8 +190,17 @@ the operands is equal.
 Internally, all network objects are represented in 128 bit format.
 The numeric representation of the network is compared through the
 corresponding operation. Comparisons are tried first on the address portion
-of the object and if that is equal then the cidr portion of the masks are
-compared.
+of the object and if that is equal then the NUMERIC cidr portion of the
+masks are compared. This leads to the counterintuitive result that
+
+        /24 > /16
+
+Comparision should not be done on netaddr objects with different CIDR as
+this may produce indeterminate - unexpected results,
+rather the determination of which netblock is larger or smaller should be
+done by comparing 
+
+	$ip1->masklen <=> $ip2->masklen
 
 =item B<Addition of a constant>
 
@@ -342,10 +357,16 @@ sub do_prefix ($$$) {
 
 =item C<-E<gt>new6([$addr, [ $mask]])>
 
-These methods creates a new address with the supplied address in
+=item C<-E<gt>new_from_aton($netaddr)>
+
+The first two methods create a new address with the supplied address in
 C<$addr> and an optional netmask C<$mask>, which can be omitted to get
 a /32 or /128 netmask for IPv4 / IPv6 addresses respectively
 
+B<new_from_aton> takes a packed IPv4 address and assumes a /32 mask. This
+function replaces the DEPRECATED :aton functionality which is fundamentally
+broken.
+
 C<-E<gt>new6> marks the address as being in ipV6 address space even if the
 format would suggest otherwise.
 
@@ -621,7 +642,42 @@ sub short($) {
   return _compV6($addr);
 }
 
-=pod
+=item C<-E<gt>full()>
+
+Returns the address part in FULL notation for 
+ipV4 and ipV6 respectively.
+
+  i.e. for ipV4
+    0000:0000:0000:0000:0000:0000:127.0.0.1
+
+       for ipV6
+    0000:0000:0000:0000:0000:0000:0000:0000
+
+To force ipV4 addresses into full ipV6 format use:
+
+=item C<-E<gt>full6()>
+
+Returns the address part in FULL ipV6 notation
+
+=cut
+
+sub full($) {
+  if (! $_[0]->{isv6} && isIPv4($_[0]->{addr})) {
+    my @hex = (unpack("n8",$_[0]->{addr}));
+    $hex[9] = $hex[7] & 0xff;
+    $hex[8] = $hex[7] >> 8;
+    $hex[7] = $hex[6] & 0xff;
+    $hex[6] >>= 8;
+    return sprintf("%04X:%04X:%04X:%04X:%04X:%04X:%d.%d.%d.%d",@hex);
+  } else {
+    &full6;
+  }
+}
+
+sub full6($) {
+  my @hex = (unpack("n8",$_[0]->{addr}));
+  return sprintf("%04X:%04X:%04X:%04X:%04X:%04X:%04X:%04X",@hex);
+}
 
 =item C<$me-E<gt>contains($other)>
 
@@ -734,12 +790,17 @@ Note that C<$me> must be a C<NetAddr::IP> object.
 =cut
 
 sub compactref($) {
-  my @r = sort @{$_[0]}
-	or return [];
-  return [] unless @r;
-  foreach(0..$#r) {
-    $r[$_]->{addr} = $r[$_]->network->{addr};
+#  my @r = sort { NetAddr::IP::Lite::comp_addr_mask($a,$b) } @{$_[0]}		# use overload 'cmp' function
+#	or return [];
+#  return [] unless @r;
+
+  return [] unless (my @unr = @{$_[0]});
+
+  foreach(0..$#unr) {
+    $unr[$_]->{addr} = $unr[$_]->network->{addr};
   }
+  my @r = sort @unr;
+
   my $changed;
   do {
 	$changed = 0;
@@ -1059,7 +1120,7 @@ __END__
 
 =head1 HISTORY
 
-$Id: IP.pm,v 4.7 2007/06/06 20:43:38 luisemunoz Exp $
+$Id: IP.pm,v 4.8 2008/06/07 20:43:38 luisemunoz Exp $
 
 =over
 
@@ -1691,6 +1752,33 @@ ipV4 and ipV6 notations can be freely mixed and matched.
 Additional methods added to force operations into ipV6 space even when 
 ipV4 notation is used.
 
+=item 4.05
+
+	NetAddr::IP :aton	DEPRECATED !
+	new method "new_from_aton"
+
+THE FOLLOWING CHANGES MAY BREAK SOME CODE !
+
+      Inherited methods from Lite.pm updated as follows:
+
+        comparisons of the form <, >, <=, >=  
+
+                10.0.0.0/24 {operator} 10.0.0.0/16
+
+        return now return the comparision of the cidr value
+        when the address portion is equal.   
+        Thanks to Peter DeVries for spotting this bug.
+
+        ... and leading us to discover that this next fix is required
+
+        comparisons of the form <=>, cmp
+        now return the correct value 1, or -1
+        when the address portion is equal and the CIDR value is not
+        i.e.    where /16 is > /24, etc...
+
+	This is the OPPOSITE of the previous return values for
+	comparison of the CIDR portion of the address object
+
 =back
 
 =head1 AUTHORS
@@ -1706,7 +1794,7 @@ so by using it you accept any and all the liability.
 =head1 LICENSE
 
 This software is (c) Luis E. Muñoz, 1999 - 2007, and (c) Michael
-Robinton, 2006 - 2007.  It can be used under the terms of the Perl
+Robinton, 2006 - 2008.  It can be used under the terms of the Perl
 artistic license provided that proper credit for the work of the
 authors is preserved in the form of this copyright notice and license
 for this module.
